using System;
using System.IO;
using System.Text;
using System.Security.Permissions;
using System.Diagnostics;
using Microsoft.Win32;
using System.Security;
using System.Collections;

using com.macromedia.mesh.log;
using com.macromedia.mesh.Flash;
using com.macromedia.mesh.utils;

		//these go in exe wrapper.
		//log file path // should these be in the wrapper class?
		//-v verbose //should this be above?
		//-o setOutputFile //should default be here or above?
		//-t setTempDirectory // should default be here or above
		//-b build comment. maybe make this a path to an actionscript file that gets included?

//this subscribes to the Logable interface. 

namespace com.macromedia.mesh.Flash
{
	public class FlashCommand
	{
		private Process flash;

		private FileInfo sourceFile;
		private FileInfo outputFile;
		private FileInfo flashExecutable;
		private FileInfo tempFile;
		private FileInfo flashLogFile;
        private ArrayList classpath;
		private string requestId;

		private bool export = true;
		private bool saveAndCompact = false;
		private bool closeFlashAfterComplete = true;
		private bool publish = false;

		//right now this is false, because flash prompts on save.
		private bool closeDocAfterComplete = false;

        private int flashAuthoringVersion;

        private string oldClasspath;

		public FlashCommand()
		{
			flash = new Process();

			requestId = (Guid.NewGuid()).ToString();
		}

		public Process Start()
		{
			if(!DoAction())
			{
				//Action not specified exception
				throw new ActionNotSpecifiedException("No action specified.");
			}

			if(this.flashExecutable == null)
			{
				throw new Exception("Executable not specified.");
			}

			StreamWriter sw;
			try
			{
				UTF8Encoding utf8 = new UTF8Encoding(true);
				sw = new StreamWriter(tempFile.FullName, false, utf8);
				sw.AutoFlush = true;
				string aSource = GetJSFLSource();
				sw.Write(aSource);
				sw.Close();
			}
			catch(Exception e)
			{
				throw new Exception("An error occured while creating jsfl source file at : " + tempFile.FullName, e);
			}

			//should we delete the jsfl file?

			flash.StartInfo.FileName = flashExecutable.FullName;

			//move this to a parameter
			flash.StartInfo.Arguments = "\"" + tempFile.FullName + "\"";

            setNewFlashClasspath();
			flash.Start();

			return flash;
		}

        private void setNewFlashClasspath()
        {
            RegistryKey flashKey = getClasspathKey();
            if (flashKey == null)
            {
                throw new Exception("There is no classpath key found :(");
            }
            string currentClasspath = (string)flashKey.GetValue("Package Paths");
            oldClasspath = currentClasspath;
            string newValue = currentClasspath;
            if (classpath != null)
            {
                for (int i = 0; i < classpath.Count; i++)
                {
                    if (currentClasspath.IndexOf(((DirectoryInfo)classpath[i]).FullName) == -1)
                    {
                        newValue = newValue + ";" + ((DirectoryInfo)classpath[i]).FullName;
                    }
                }
            }
            flashKey.SetValue("Package Paths", newValue);
        }

        private RegistryKey getClasspathKey()
        {
            string flashPath = @"Software\Macromedia";
            RegistryKey flashKey = Registry.CurrentUser.OpenSubKey(flashPath, true);
            string[] versions = flashKey.GetSubKeyNames();
            foreach (string versionName in versions)
            {
                if (versionName.IndexOf("Flash") >= 0)
                {
                    string versionString = versionName.Substring(versionName.Length - 1);
                    int versionNumber;
                    try
                    {
                        versionNumber = int.Parse(versionString);
                        if (flashAuthoringVersion == versionNumber)
                        {
                            flashKey = flashKey.OpenSubKey(versionName, true);
                            flashKey = flashKey.OpenSubKey("ActionsInspector", true);
                            return flashKey;
                        }
                    }
                    catch (Exception)
                    {
//                        Console.WriteLine("Not a number: " + versionString);
                    }
                }
            }
            return null;
        }

        public void restoreOldClasspath()
        {
            RegistryKey flashKey = getClasspathKey();
            if (flashKey == null)
            {
                throw new Exception("There is no classpath key found :(");
            }
            flashKey.SetValue("Package Paths", oldClasspath);
        }

        public void setFlashVersion (int flashVersion)
        {
            flashAuthoringVersion = flashVersion;
        }

		private bool DoAction()
		{
			return (export || saveAndCompact || publish);
		}

		public void SetTempDirectory(DirectoryInfo di)
		{
			if(!di.Exists)
			{
				di.Create();
			}

			tempFile = new FileInfo(di.FullName + "\\" + GetTempFileName());
			flashLogFile = FileUtils.ChangeExtension(tempFile, "log");
		}

		private string GetTempFileName()
		{
			return requestId + ".jsfl";
		}

		public void CleanUp()
		{
			if(tempFile != null && tempFile.Exists)
			{
				tempFile.Delete();
			}

			if(flashLogFile != null && flashLogFile.Exists)
			{
				flashLogFile.Delete();
			}

            restoreOldClasspath();
		}

		public void SetOutputFilePath(string outputFilePath)
		{
			this.outputFile = new FileInfo(outputFilePath);
		}

		public void SetSourceFilePath(string sourceFilePath)
		{
			SetSourceFile(new FileInfo(sourceFilePath));
		}

		public void SetSourceFile(FileInfo f)
		{

			if(!f.Exists)
			{
				throw new FileNotFoundException("The source file specified does not exist.");
			}

			sourceFile = f;
		}

		public FileInfo GetFlashLogFile()
		{
			return flashLogFile;
		}

		public string GetOutput()
		{
//Console.WriteLine("Flash Log File :" + GetFlashLogFile().FullName);
			flashLogFile.Refresh();
			if(!GetFlashLogFile().Exists)
			{

//Console.WriteLine("Does not exists.");
				return "";
			}

			StreamReader sr = flashLogFile.OpenText();
			string output = sr.ReadToEnd();
			sr.Close();

			return output;
		}

		public string GetJSFLSource()
		{
			StringBuilder sb = new StringBuilder();
			sb.Append("var sourceFile = \"" + FileUtils.GetURI(sourceFile) + "\";\n");
			sb.Append("var outputFile = \"" + FileUtils.GetURI(outputFile) + "\";\n");

			sb.Append("var logFile = \"" + FileUtils.GetURI(flashLogFile) + "\";\n");

			sb.Append("var doc = fl.openDocument(sourceFile);\n");

			if(saveAndCompact)
			{
				sb.Append("doc.saveAndCompact(false);\n");
			}

			//			if(buildComment != null)
			//			{
			//				//need to escape build comment.
			//				sb.Append("var actionScript = \"_global.BuildInfo = {comment:\\\""+buildComment+"\\\"};\";\n");
			//				sb.Append("var oldAs = fl.getDocumentDOM().getTimeline().getFrameProperty(\"actionScript\");\n");
			//				sb.Append("fl.getDocumentDOM().getTimeline().setFrameProperty(\"actionScript\", actionScript + \"\\n\" + oldAs);\n");
			//			}

			sb.Append("fl.outputPanel.clear();\n");

			if(publish)
			{
				sb.Append("fl.getDocumentDOM().publish();");
			}

			if(export)
			{
                sb.Append("fl.getDocumentDOM().exportSWF(outputFile, true);\n");
			}

			sb.Append("fl.outputPanel.save(logFile, true);\n");

			if(closeFlashAfterComplete)
			{
				sb.Append("fl.quit(false);\n");
			}
			else
			{
				if (closeDocAfterComplete)
				{
					sb.Append("fl.getDocumentDOM().close(false);\n");
				}
			}

			return sb.ToString();
		}

		public void SetFlashExectuablePath(string flashPath)
		{
			SetFlashExecutableFile(new FileInfo(flashPath));
		}

        public void SetClasspath (string inputClasspath)
        {
            string[] classpaths;
            if (inputClasspath.IndexOf(";") == -1)
            {
                classpaths = new string[] { inputClasspath };
            }
            else
            {
                Char[] splitArray = new Char[] { ';'};
                classpaths = inputClasspath.Split(splitArray);
            }
            foreach (string i in classpaths)
            {
                if (classpath == null)
                {
                    classpath = new ArrayList();
                }
                DirectoryInfo aClasspath = new DirectoryInfo (i);
                if (!aClasspath.Exists)
                {
                    ////ClasspathDoesNotExistException
                    throw new Exception("The classpath catalog specified does not exist.");
                }
                else
                {
                    classpath.Add(aClasspath);
                }
            }
        }

        public void SetFlashExecutableFile(FileInfo f)
		{
			if(!f.Exists)
			{
				////FlashExecutableDoesNotExistException
				throw new Exception("The flash executable specified does not exist.");
			}

			flashExecutable = f;
		}

		public bool Export
		{
			set
			{
				export = value;
			}
			get
			{
				return export;
			}
		}


		public bool SaveAndCompact
		{
			set
			{
				saveAndCompact = value;
			}
			get
			{
				return saveAndCompact;
			}
		}

		public bool CloseDocAfterComplete
		{
			set
			{
				closeDocAfterComplete = value;
			}
			get
			{
				return closeDocAfterComplete;
			}
		}

		public bool CloseFlashAfterComplete
		{
			set
			{
				closeFlashAfterComplete = value;
			}
			get
			{
				return closeFlashAfterComplete;
			}
		}

		public bool Publish
		{
			set
			{
				publish = value;
			}
			get
			{
				return publish;
			}
		}

	}
}
