using System;
using System.Collections;
using System.Collections.Specialized;
using Microsoft.Win32;

namespace com.macromedia.mesh.Flash
{
	/// <summary>
	/// Summary description for FlashUtils.
	/// </summary>
	public class FlashInfo
	{
		
		private ListDictionary versionTable;
		private int currentVersion;

		public FlashInfo()
		{
			versionTable = FlashInfo.CreateFlashVersionTable();
			currentVersion = FlashInfo.GetMostCurrentVersion(versionTable);
		}

		public ListDictionary FlashVersionTable
		{
			get
			{
				return versionTable;
			}
		}

		public int CurrentVersion
		{
			get
			{
				return currentVersion;
			}
		}

		//thanks to Robin Debruil for the method.
		public static ListDictionary CreateFlashVersionTable()
		{
            ListDictionary flashVersionTable = new ListDictionary();

//			string flashPath = @"SOFTWARE\Macromedia\Flash";
//			RegistryKey flashKey = Registry.LocalMachine.OpenSubKey(flashPath);
//			string[] versions = flashKey.GetSubKeyNames();
//
//			foreach(string vn in versions)
//			{
//				RegistryKey vk = flashKey.OpenSubKey(vn + @"\Installation");
//				string exePath = (string)vk.GetValue("InstallPath");
//
//				flashVersionTable.Add(int.Parse(vn), exePath );
//			}

			string flashPath = @"SOFTWARE\Macromedia\Flash";
			RegistryKey flashKey = Registry.LocalMachine.OpenSubKey(flashPath);
            string[] versions = flashKey.GetSubKeyNames();
            foreach (string vn in versions)
			{
                RegistryKey vk = flashKey.OpenSubKey(vn + @"\Installation");
				if(vk != null)
				{
					string exePath = (string)vk.GetValue("InstallPath");
					if(exePath != null)
					{
                        try
                        {
                            int flashVersionNumber = int.Parse(vn);
                            flashVersionTable.Add(flashVersionNumber, exePath);
                        }
                        catch (ArgumentException)
                        {
                            // Nothing
                        }
					}
				}
			}

			return flashVersionTable;
		}

		public string GetMostCurrentVersionPath()
		{
			return (string)versionTable[currentVersion];
		}

		public string GetVersionPath(int version)
		{
			///
			/// TODO : thorw exception is version doesnt exist. 
			///
			return (string)versionTable[version];
		}

		public static int GetMostCurrentVersion(ListDictionary versionTable)
		{
			if(versionTable.Count < 1)
			{
				throw new FlashNotFoundException("The path to the Flash executable could not be found.");
			}

			int top = -1;
			ICollection keys = versionTable.Keys;

			foreach(int key in keys)
			{
				if(key > top)
				{
					top = key;
				}
			}

			return top;
		}

	}
}
