using System;
using System.Diagnostics;
using System.Threading;
using System.Collections;
using System.IO;
using System.Reflection;

using System.Windows.Forms;

using com.macromedia.mesh.utils;
using com.macromedia.mesh.log;

//fix line returns in flash output log file output.
//add option to specify publish setting
//add option to specify publish details.
//add option to specify different jsfl file (create jsfl file and then pass it to flash command object).
//		i.e. you could specify a template with tokens, and then pass in name / value pairs to replace in the file.

using CommandLine.Utility;

namespace com.macromedia.mesh.Flash
{
	/// <summary>
	/// Creates an executable that wraps the FlashCommand class.
	/// </summary>
	public class FlashCommandRunner
	{

		private bool verbose = false;
		private bool keepTempFile = false;
		private bool closeFlashAfterComplete = false;
		private bool quiet = false;

		private DirectoryInfo executableDir;

		private LoggerManager logManager;
		private Arguments arguments;
		private int timeout = 60;

		private string sourceFilePath;

        private int flashAuthoringVersion;

		public FlashCommandRunner(Arguments arguments)
		{
			this.arguments = arguments;
		}

		public DirectoryInfo ExecutableDirectory
		{
			set
			{
				executableDir = value;
			}
		}

		public bool Verbose
		{
			set
			{
				verbose = value;
			}
		}

		public bool Quiet
		{
			set
			{
				quiet = value;
			}
		}

		public void SetLoggerManager(LoggerManager logManager)
		{
			this.logManager = logManager;
		}

		//-s
		private string DetermineSourceFile(Arguments arguments)
		{

			if(arguments.ParamExists("s"))
			{
				if(!arguments.ParamHasValue("s"))
				{
					throw new InvalidCommandLineArgumentException("Source setting must have a value : -s SOURCEFILEPATH");
				}

				LogVerboseMessage("Using Source File : " + arguments["s"]);
				return arguments["s"];
			}
			else
			{
				throw new InvalidCommandLineArgumentException("Source file must be specified.");
			}
		}

		//-d
		private DirectoryInfo DetermineTempDirectory(Arguments arguments)
		{
			DirectoryInfo di;
			if(arguments.ParamExists("d"))
			{
				if(!arguments.ParamHasValue("d"))
				{
					throw new InvalidCommandLineArgumentException("Temp directory path setting must have a value : -d TEMPDIRPATH");
				}

				di = new DirectoryInfo(arguments["d"]);
			}
			else
			{
				di = new DirectoryInfo(Path.GetTempPath());
			}

			LogVerboseMessage("Using Temp Directory : " + di.FullName);
			return di;
		}

		private string DetermineOutputFile(Arguments arguments, string sourceFile)
		{
			string path;
			if(arguments.ParamExists("o"))
			{
				if(!arguments.ParamHasValue("o"))
				{
					throw new InvalidCommandLineArgumentException("Output file setting must have a value : -o OUTPUTFILE");
				}
				
				path = arguments["o"];

			}
			else
			{
				path = FileUtils.ChangeExtension(sourceFile, "swf");
			}

			FileUtils.CreateDirectoryFromFileName(path);

			LogVerboseMessage("Output File : " + path);
			return path;
		}

        // -cp
        private string DetermineFlashClasspath (Arguments arguments)
        {
            string aPath;
            if (arguments.ParamExists("cp"))
            {
                if (!arguments.ParamHasValue("cp"))
                {
                    throw new InvalidCommandLineArgumentException("Classpath setting must have a value : -cp CLASSPATH");
                }

                aPath = arguments["cp"];
            }
            else
            {
                return null;
            }
            return aPath;
        }

		//-f
		private string DetermineFlashExecutablePath(Arguments arguments)
		{
			string path;
			if(arguments.ParamExists("f"))
			{
				if(!arguments.ParamHasValue("f"))
				{
					throw new InvalidCommandLineArgumentException("Flash Path setting must have a value : -f FLASHPATH");
				}

				path = arguments["f"];
                if (path.IndexOf("2004") == -1)
                {
                    flashAuthoringVersion = 8;
                }
                else
                {
                    flashAuthoringVersion = 7;
                }
			}
			else
			{
				FlashInfo fi = new FlashInfo();
				path = fi.GetMostCurrentVersionPath() + "\\flash.exe";
                flashAuthoringVersion = fi.CurrentVersion;
			}

			return path;
		}

		private int DetermineTimeout(Arguments arguments)
		{
			int timeout;
			if(arguments.ParamExists("t"))
			{
				if(!arguments.ParamHasValue("t"))
				{
					throw new InvalidCommandLineArgumentException("Timeout setting must have a value : -t TIMEOUTVALUE");
				}

				try
				{
					timeout = int.Parse(arguments["t"]);
				}
				catch(Exception)
				{
					throw new InvalidCommandLineArgumentException("Timeout setting must be a valid number. : -t TIMEOUTVALUE");
				}
			}
			else
			{
				timeout = this.timeout;
			}

			return timeout;

		}

		private bool DetermineCloseFlashAfterComplete(Arguments arguments)
		{
			return(arguments.ParamExists("x"));
		}

		private void Run()
		{
			FlashCommand fc = new FlashCommand();

			//-s : specifies source files. Required.
			sourceFilePath = DetermineSourceFile(arguments);
			fc.SetSourceFilePath(sourceFilePath);

            //-cp : specifies classpath. Optional.
            string aClasspath = DetermineFlashClasspath(arguments);
            if (aClasspath != null)
            {
                fc.SetClasspath(DetermineFlashClasspath(arguments));
            }

			//-d : specifies temp directory. Optional.
			fc.SetTempDirectory(DetermineTempDirectory(arguments));
			
			//-o : output file. Optional.
			fc.SetOutputFilePath(DetermineOutputFile(arguments, sourceFilePath));

			//-f : specifies Flash executable path. Optional
			fc.SetFlashExectuablePath(DetermineFlashExecutablePath(arguments));

            fc.setFlashVersion(flashAuthoringVersion);

			//-t : specifies timeout value. Optional
			timeout = DetermineTimeout(arguments);

			//-e : Specifies export action.
			fc.Export = (arguments.ParamExists("e"));

			//-c : Specifies save and compact action.
			fc.SaveAndCompact = (arguments.ParamExists("c"));
			
			//-x : Specifies whether Flash should be closed after it is done processing its file. Optional.
			closeFlashAfterComplete = this.DetermineCloseFlashAfterComplete(arguments);
			fc.CloseFlashAfterComplete = closeFlashAfterComplete;

			// -y : Specifies whether the FLA should be closed once it the actions have been performed.
			fc.CloseDocAfterComplete = (arguments.ParamExists("y"));

			//-k : Specifies whether temp jsfl file should be deleted. Optional.
			keepTempFile = arguments.ParamExists("k");

			//-p : whether the file should be published
			fc.Publish = arguments.ParamExists("p");

			LogVerboseMessage("Starting Flash Process.");

			TimeSpan startTime = new TimeSpan(DateTime.Now.Ticks);
			Process fp = fc.Start();

			try
			{
				if(timeout > 0)
				{
					WaitForProcessExit(fp, timeout, fc.GetFlashLogFile());
				}
			}
			catch(TimeOutException)
			{
                fc.restoreOldClasspath();
				throw new TimeOutException("\nThe Flash process timed out and did not exit. Temp files may not have been cleaned up.");
			}

			TimeSpan elapsedTime = (new TimeSpan(DateTime.Now.Ticks)) - startTime;

			LogVerboseMessage("Total Process Time : " + (elapsedTime.Minutes + "m:" + elapsedTime.Seconds + "s:" + 
									elapsedTime.Milliseconds + "ms"));

			string output = fc.GetOutput();

			if(output.Length > 0)
			{
				LogMessage("\n\r----------------------- Flash Output --------------------\n\r" + fc.GetOutput());
				LogMessage("----------------------- End Output -----------------------");
			}

			//note, we need to make sure flash has loaded before we delete the tmp file.
			LogVerboseMessage("Cleaning up temp files.");
			if(!keepTempFile)
			{
				fc.CleanUp();
			}

			LogVerboseMessage("Exiting.");
			return;
		}

		private void WaitForProcessExit(Process process, int timeout, FileInfo fi)
		{
			int elapsed = 0;

			while(elapsed < timeout)
			{
				Console.Write(".");
				fi.Refresh();
				
				if(/*process.HasExited || */File.Exists (fi.FullName))
				{
					Console.WriteLine();
					return;
				}

				Thread.Sleep(1 * 1000);
				elapsed ++;
			}
			
			Console.WriteLine();
			throw new Exception("Timeout waiting for Process to exit.");
		}

		public static string GetUsage()
		{
			return "Usage : FlashCommand.exe -e | -c | -p [-q] [-v] [-x] [-q] (-s <sourcefile>) ([-l] [<logfile>]) ([-o] <exportpath>) ([-f] <flashpath>) ([-t] <timeout>) ([-d] <tempdirectory>)([-cp] <\"classpath1;classpath2;classpath3\">)";
		}

		public static string GetHelp()
		{
			return FlashCommandRunner.GetUsage();
		}

		public static string GetVersion()
		{
			Assembly a = Assembly.GetAssembly(Type.GetType("com.macromedia.mesh.Flash.FlashCommandRunner"));
			Version v = a.GetName().Version;
			string version = v.Major + "," + v.Minor + "," + v.Build + "," + v.Revision;

			return "FlashCommand Version : " + version + "\nCreated by Mike Chambers\nMinor update by Constantiner";
		}

		private void LogMessage(string message)
		{
			if((logManager != null))
			{
				logManager.WriteLine(message);
			}
		}

		private void LogVerboseMessage(string message)
		{
			if(verbose)
			{
				LogMessage(message);
			}
		}

		/// <summary>
		/// The entry point for the executable that parses and validates the commandline arguments, and presents a commandline
		/// interface for the user.
		/// </summary>
		[STAThread]
		public static void Main(string[] args)
		{

			DirectoryInfo executableDir = new DirectoryInfo(Path.GetDirectoryName(Application.ExecutablePath));
			DirectoryInfo logDirectory = new DirectoryInfo(executableDir + "\\log");

			if(!logDirectory.Exists)
			{
				logDirectory.Create();
			}

			Arguments arguments = new Arguments(args);

			//-help : displays usage information.
			if(arguments.ParamExists("help"))
			{
				Console.Out.WriteLine(FlashCommandRunner.GetUsage());
				return;
			}

			//-version : displays information about the current build of the program.
			if(arguments.ParamExists("version"))
			{
				Console.Out.WriteLine(FlashCommandRunner.GetVersion());
				return;
			}

			FlashCommandRunner fcr = new FlashCommandRunner(arguments);

			LoggerManager lm = new LoggerManager();

			try
			{
				//log file path
				if(arguments.ParamExists("l"))
				{
					FileLogger fl;
					if(arguments.ParamHasValue("l"))
					{
						fl = new FileLogger(arguments["l"], true);
					}
					else
					{
						fl = new FileLogger(logDirectory.FullName + "\\flashcommand.log", true);
					}

					lm.Add(fl);
				}


				//-q : sets quiet mode. No output will be displayed at all. This includes errors. This only applies to the
				//console, and does not apply to the log file, if a log file is specified.
				if(!arguments.ParamExists("q"))
				{
					ConsoleLogger cl = new ConsoleLogger();
					cl.TimeStampFormat = "";

					lm.Add(cl);
				}
				else
				{
					lm.WriteLine("Enabling quiet mode. No output will be sent to command line.");
					fcr.Quiet = true;
				}

				//-v : sets verbose mode. If specified, detailed information will be output to the console, log file or both
				//(depending on the settings.)
				if(arguments.ParamExists("v"))
				{
					lm.WriteLine("Enabling Verbose mode.");
					fcr.Verbose = true;
				}

				fcr.SetLoggerManager(lm);
				fcr.Run();

			}
			catch(InvalidCommandLineArgumentException iclae)
			{
				lm.WriteLine("Invalid Command Line : " + iclae.Message);
				lm.WriteLine(FlashCommandRunner.GetUsage());
			}
			catch(ActionNotSpecifiedException anse)
			{
				lm.WriteLine("Invalid Command Line : " + anse.Message);
				lm.WriteLine(FlashCommandRunner.GetUsage());
			}
			catch(FlashNotFoundException fnfe)
			{
				lm.WriteLine("Invalid Command Line : " + fnfe.Message);
			}
			catch(FileNotFoundException fnfe)
			{
				lm.WriteLine("Invalid Command Line : " + fnfe.Message);
			}
			catch(Exception e)
			{
				lm.WriteLine("\nAn Error occured : " + e.Message + "\n" + e.StackTrace);
			}
			finally
			{
				lm.Flush();
				lm.Close();
			}
		}
		
	}
}
