using System;
using System.IO;
using System.Text;

using System.Runtime.InteropServices;

using Microsoft.Win32;

namespace com.macromedia.mesh.Flash.FlashCustomSetupActions
{

	public enum FuFlags : long
	{
		SMTO_NORMAL         = 0x00,
		SMTO_BLOCK          = 0x01,
		SMTO_ABORTIFHUNG    = 0x02
	}

	public enum Win32h : uint
	{
		SMTO_NORMAL         = 0x00,
		SMTO_BLOCK          = 0x01,
		SMTO_ABORTIFHUNG    = 0x02,
		WM_SETTINGCHANGE    = 0x1A,
		HWND_BROADCAST      = 0xFFFF
	}

	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class FlashCustomSetupActions
	{

		[DllImport("user32.dll", CharSet=CharSet.Auto)]
		public static extern uint SendMessageTimeout(
			uint hwnd,               // HWND_BROADCAST == all top level
			uint msg,                // WM_SETTINGCHANGE
			IntPtr wParam,           // null
			StringBuilder lParam,    // null probably ok here, or "Environment"
			uint fuFlags,            // SMTO_ABORTIFHUNG
			IntPtr uTimeout,         // 5000 == 5 seconds
			IntPtr lpdwResult        // return value
			);

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			if(args.Length != 2)
			{
				Console.WriteLine("Not enough args.");
				return;
			}

			string installPath = args[1];

			FlashCustomSetupActions fcsa;
			try
			{
				fcsa = new FlashCustomSetupActions(installPath);
			}
			catch(Exception)
			{
				Console.WriteLine("Install Error");
				return;
			}

			if(args[0].Equals("/INSTALL"))
			{
				fcsa.Install();
			}
			
			if(args[0].Equals("/UNINSTALL"))
			{
				fcsa.Uninstall();
			}

		}

		private string path;
		private string installPath;
		private RegistryKey rk;
		private const string PATHSEPERATER = ";";

		public FlashCustomSetupActions(string installPath)
		{
			this.installPath = installPath;

			rk = Registry.CurrentUser.OpenSubKey("Environment", true);
			path = (string)rk.GetValue("Path");

			Console.WriteLine("Path : " + path);
		}

		private void Install()
		{
			if(KeyExists(installPath, path))
			{
				return;
			}

			string delim = "";

			if(!path.EndsWith(PATHSEPERATER))
			{
				delim = PATHSEPERATER;
			}

			string newPath = path + delim + installPath; 
			Console.WriteLine("Writing : " + newPath);

			WritePath(newPath);

			return;
		}

		private void WritePath(string newPath)
		{
			rk.SetValue("Path", newPath);
			rk.Flush();
			BroadcastPathChange();
		}

		private void Uninstall()
		{
			if(!KeyExists(installPath, path))
			{
				return;
			}

			string[] pathArr = GetPathArray();
			int len = pathArr.Length;

			if(len == 0)
			{
				return;
			}

			StringBuilder newPath = new StringBuilder();

			for(int i = 0; i < len; i++)
			{

				if(installPath.ToLower().Trim().Equals(pathArr[i].ToLower().Trim()))
				{
					Console.WriteLine("Found match : omitting.");
					continue;
				}

				newPath.Append(pathArr[i]);
				newPath.Append(PATHSEPERATER);
			}

			Console.WriteLine("Writing : " + newPath.ToString());

			WritePath(newPath.ToString());
		}

		private bool KeyExists(string installPath, string path)
		{
			string[] pathArr = GetPathArray();
			int len = pathArr.Length;

			for(int i = 0; i < len; i++)
			{
				if(installPath.ToLower().Trim().Equals(pathArr[i].ToLower().Trim()))
				{
					Console.WriteLine("Found : Key. Exiting");
					return true;
				}
				Console.WriteLine(i + " : " + pathArr[i]);
			}

			return false;
		}

		private string[] GetPathArray()
		{
			char[] d = new char[1];
			d[0] = PATHSEPERATER.ToCharArray()[0];

			return path.Split(d);
		}

		private void BroadcastPathChange()
		{
			IntPtr timeout = new IntPtr(5000L);
			IntPtr result = new IntPtr(0);
			StringBuilder env = new StringBuilder("Environment");

			SendMessageTimeout
				(
				(uint) Win32h.HWND_BROADCAST,
				(uint) Win32h.WM_SETTINGCHANGE,
				IntPtr.Zero,
				env,
				(uint) Win32h.SMTO_ABORTIFHUNG,
				timeout,
				result
				);
		}
	}
}
