using System;
using System.Text;
using System.Globalization;

namespace com.macromedia.mesh.log
{
	/// <summary>
	/// Summary description for Logger.
	/// </summary>
	abstract public class Logger
	{
		protected string NEWLINE = System.Environment.NewLine;
		protected CultureInfo ci;
		protected string timeStampFormat;

		public Logger()
		{
			CultureInfo = new CultureInfo( "en-US", false );

			DateTimeFormatInfo di = ci.DateTimeFormat;
			timeStampFormat = di.ShortDatePattern + " " + di.LongTimePattern;
		}

		abstract public void Write(string msg);
		abstract public void WriteLine(string msg);
		abstract public void Close();
		abstract public void Flush();

		/// <param name="CultureInfo">The CultureInfo object which is used to determine the default formatting for the date, time and other strings.</param>
		public CultureInfo CultureInfo
		{
			get
			{
				return ci;
			}
			set
			{
				ci = value;
			}
		}

		public string TimeStampFormat
		{
			get
			{
				return timeStampFormat;
			}
			set
			{
				timeStampFormat = value;
			}
		}

		protected string GetDateTimeStamp(string formatString)
		{
			if(formatString.Length < 1)
			{
				return "";
			}

			string ts = DateTime.Now.ToString(formatString) + " : ";

			return ts;
		}

		protected string GetDateTimeStamp()
		{
			return GetDateTimeStamp(timeStampFormat);
		}

	}
}
